from time import sleep
import selenium.common.exceptions

from .attend import AttendMeet

class GoogleMeet(AttendMeet):
    def __init__(self, **kwargs):
        """
        Parameters:\n
            kwargs['code'] (string): Meeting code\n
        """
        super().__init__(**kwargs)
        self.login_url = "google"

    def enter_class(self):
        try: self.driver.get(self.meet_url)
        except selenium.common.exceptions.InvalidArgumentException:
            print("ERROR ****** Meeting code was not properly set. Please, provide a valid one and try again! ******")
        except selenium.common.exceptions.InvalidSessionIdException:
            return

        for _ in range(25):
            try:
                self.driver.find_element_by_class_name("uArJ5e UQuaGc Y5sE8d uyXBBb xKiqt M9Bg4d".replace(" ", ".")).click()
                break
            except selenium.common.exceptions.NoSuchElementException:
                sleep(1)
                continue

        return

    def doLogin(self):
        self.driver.get(self.login_url)

        #USER
        try:
            self.driver.find_element_by_id("identifierId").send_keys(self.login_data["user"])
            self.driver.find_element_by_id("identifierNext").click()
            try:
                if self.driver.find_element_by_class_name("o6cuMc"):
                    print("ERROR ****** Login failed. Check user and try again! ******")
                    return
            except selenium.common.exceptions.NoSuchElementException: pass
        except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementClickInterceptedException):
            print("ERROR ****** Login failed. Check your connection and try again! ******")
            return
            
        #PASSWORD
        for _ in range(15):
            try:
                self.driver.find_element_by_name("password").send_keys(self.login_data["passwd"])
                break
            except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementNotInteractableException):
                sleep(1)
        try:
            self.driver.find_element_by_id("passwordNext").click()
            try:
                if self.driver.find_element_by_class_name("EjBTad"):
                    print("ERROR ****** Login failed. Check your password and try again! ******")
                    return
            except selenium.common.exceptions.NoSuchElementException: pass
        except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementClickInterceptedException):
                print("ERROR ****** Login failed. Check your connection and try again! ******")
                return

    def set_meeting_url(self, code):
        code_len = len(code)
        mCode = ""

        if "https://meet.google.com/" in code:
            self.meet_url = code
        elif "meet.google.com/" in code:
            self.meet_url= "{0}{1}".format("https://", code)
        else:
            if type(code) != type(0) and ((code_len == 12 and code[3] == "-" and code[8] == "-") or code_len == 10):
                for crc in code:
                    if type(crc) == type(0):
                        print("ERROR ****** Meeting code must not contain numbers! ******")
                    else:
                        mCode = code
            else: print("ERROR ****** Meeting code not accepted! Please check again ******")

            self.meet_url="https://meet.google.com/%s" % mCode
        return



from abc import ABC, abstractmethod
from time import sleep

import selenium.common.exceptions

class CreateMeet(ABC):
    def __init__(self, driver = None,**kwargs):
        self.__code = None
        self.__driver = driver

    @abstractmethod
    def new_class(self):
        return

    def set_new_class(self, code):
        self.__code = code

    @property
    def code(self):
        try:
            return self.__code
        except AttributeError:
            print("Code unset!")
            return None

    @property
    def driver(self): return self.__driver

class CreateGoogle(CreateMeet):
    def __init__(self, driver = None, **kwargs):
        super().__init__(driver, **kwargs)

    def new_class(self):
        self.driver.get("https://meet.google.com/")
        button = self.driver.find_element_by_class_name("VfPpkd-LgbsSe VfPpkd-LgbsSe-OWXEXe-k8QpJ VfPpkd-LgbsSe-OWXEXe-dgl2Hf nCP5yc AjY5Oe cjtUbb Dg7t5c".replace(" ", "."))
        button.click()
        button = self.driver.find_element_by_class_name("VfPpkd-rymPhb-ibnC6b VfPpkd-rOvkhd-rymPhb-ibnC6b-OWXEXe-tPcied-hXIJHe".replace(" ", "."))
        button.click()

        for _ in range(5):
            try:
                self.set_new_class(self.driver.find_element_by_class_name("Hayy8b").text)
                break
            except (selenium.common.exceptions.NoSuchElementException): sleep(1)

        self.driver.get("https://meet.google.com/")



from getpass import getpass
from abc import ABC, abstractmethod

from selenium.webdriver.firefox.webdriver import WebDriver

import os
EXECUTABLE_PATH = os.environ["HOME"] + "/geckodriver"



class AttendMeet(ABC):
    def __init__(self, **kwargs):
        """
        Parameters:\n
            kwargs['code'] (string): Meeting code\n
        """
        self.__login_data = {}
        self.__login_url = None
        self.__driver = None
        self.meet_url = None

        try: self.set_meeting_url(kwargs["code"])
        except KeyError: pass


    @abstractmethod
    def doLogin(self): raise NotImplementedError

    @abstractmethod
    def enter_class(self): raise NotImplementedError

    @abstractmethod
    def set_meeting_url(self): raise NotImplementedError


    @property
    def login_data(self): return self.__login_data

    @login_data.setter
    def login_data(self, data):
        try:
            self.__login_data = {"user": data["user"], "passwd": data["passwd"]} 
        except KeyError:
            user = str(input("User: "))
            passwd = getpass("Password: ")
            self.__login_data = {"user": user, "passwd": passwd}

    @property
    def login_url(self): return self.__login_url

    @login_url.setter
    def login_url(self, platform):
        plat_login = {
            "google": "https://accounts.google.com/Login?hl=pt-BR",
            "meet": "https://accounts.google.com/Login?hl=pt-BR",
            "zoom": "https://zoom.us/google_oauth_signin"
        }
        try: self.__login_url = plat_login[platform]
        except KeyError: print("ERROR ****** ******** Platform not available! ******")

    @property
    def driver(self): return self.__driver

    @driver.setter
    def driver(self, browser):
        if browser == "firefox": self.__driver = WebDriver(executable_path=EXECUTABLE_PATH)
        else: self.__driver = None

    
